/**
 * Copyright © 2012, 2013 dr. ir. Jeroen M. Valk
 * 
 * This file is part of Badgerfish CPX. Badgerfish CPX is free software:
 * you can redistribute it and/or modify it under the terms of the GNU Lesser
 * General Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version. Badgerfish
 * CPX is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU Lesser General Public License for more details. You
 * should have received a copy of the GNU Lesser General Public License along
 * with Badgerfish CPX. If not, see <http://www.gnu.org/licenses/>.
 */

package com.googlecode.aaw.arche;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;

import org.slf4j.ILoggerFactory;
import org.slf4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public abstract class Arche {
	static private Logger logger = null;
	static protected final List<Object> components = new ArrayList<Object>();
	static private final Map<String, Integer> types = new HashMap<String, Integer>();
	static private Arche instance = null;
	private Class<?> c = null;
	private List<Integer> arguments = new ArrayList<Integer>();
	private int index = -1;

	static public final Arche getArche(Class<?> c, ILoggerFactory factory) {
		if (instance != null) {
			assert logger != null;
			return getArche(c);
		}
		logger = factory.getLogger(Arche.class.getCanonicalName());
		instance = new DefaultArche(factory);
		InputSource source;
		try {
			source = new InputSource(c.getClassLoader()
					.getResource("com/googlecode/aaw/arche/DefaultArche.xml")
					.openStream());
		} catch (IOException e) {
			throw new ArcheException(e);
		}
		DocumentBuilder db = (DocumentBuilder) components.get(5);
		Document document;
		try {
			document = db.parse(source);
		} catch (SAXException e) {
			throw new ArcheException(e);
		} catch (IOException e) {
			throw new ArcheException(e);
		}
		Element element = document.getDocumentElement();
		if (!element.getTagName().equals("arche"))
			throw new ArcheException("document element must be <arche>");
		NodeList children = element.getElementsByTagName("instance");
		for (int i = 0; i < children.getLength(); ++i) {
			element = (Element) children.item(i);
			types.put(element.getAttribute("type"), Integer.valueOf(i));
		}
		return getArche(c);
	}

	static public final Arche getArche(Class<?> c) {
		instance.initialize(c);
		return instance;
	}

	protected final int register(Object component) {
		int result = components.size();
		components.add(component);
		return result;
	}

	protected void initialize(Class<?> c) {
		this.c = c;
		arguments.clear();
		index = 0;
		String path = c.getCanonicalName().replaceAll("\\.", "/") + ".xml";
		URL resource = c.getClassLoader().getResource(path);
		if (resource == null) {
			throw new ArcheException("cannot initialize class "
					+ c.getCanonicalName() + ": resource '" + path
					+ "' not found");
		}
		InputSource source;
		try {
			source = new InputSource(resource.openStream());
		} catch (IOException e) {
			throw new ArcheException(e);
		}
		DocumentBuilder db = (DocumentBuilder) components.get(5);
		Document document;
		try {
			document = db.parse(source);
		} catch (SAXException e) {
			throw new ArcheException(e);
		} catch (IOException e) {
			throw new ArcheException(e);
		}
		Element element = document.getDocumentElement();
		if (!element.getTagName().equals("class"))
			throw new ArcheException("document element must be <class>");
		NodeList children = element.getElementsByTagName("argument");
		for (int i = 0; i < children.getLength(); ++i) {
			element = (Element) children.item(i);
			arguments.add(types.get(element.getAttribute("type")));
		}
	}

	abstract protected Object instantiate(Class<?> c, int index);

	public Object getInstance() {
		try {
			return instantiate(c, arguments.get(index++).intValue());
		} catch (IndexOutOfBoundsException e) {
			throw new ArcheException("class " + c.getCanonicalName()
					+ ": argument not declared");
		}
	}

}
