/**
 * Copyright © 2012, 2013 dr. ir. Jeroen M. Valk
 * 
 * This file is part of Badgerfish CPX. Badgerfish CPX is free software:
 * you can redistribute it and/or modify it under the terms of the GNU Lesser
 * General Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version. Badgerfish
 * CPX is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU Lesser General Public License for more details. You
 * should have received a copy of the GNU Lesser General Public License along
 * with Badgerfish CPX. If not, see <http://www.gnu.org/licenses/>.
 */

package com.googlecode.aaw.arche;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.ILoggerFactory;
import org.xml.sax.SAXException;

public class DefaultArche extends Arche {

	public DefaultArche(ILoggerFactory factory) throws ArcheException {
		super.register(factory);
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		super.register(dbf);
		SAXParserFactory spf = SAXParserFactory.newInstance();
		spf.setNamespaceAware(true);
		super.register(spf);
		TransformerFactory tf = TransformerFactory.newInstance();
		super.register(tf);
		XPathFactory xpf = XPathFactory.newInstance();
		super.register(xpf);
		try {
			super.register(dbf.newDocumentBuilder());
		} catch (ParserConfigurationException e) {
			throw new ArcheException(e);
		}
		try {
			super.register(spf.newSAXParser());
		} catch (ParserConfigurationException e) {
			throw new ArcheException(e);
		} catch (SAXException e) {
			throw new ArcheException(e);
		}
		try {
			super.register(tf.newTransformer());
		} catch (TransformerConfigurationException e) {
			throw new ArcheException(e);
		}
		XPath xp = xpf.newXPath();
		super.register(xp);
		try {
			super.register(xp.compile("*"));
			super.register(xp.compile("text()"));
		} catch (XPathExpressionException e) {
			throw new ArcheException(e);
		}
	}

	public Object instantiate(Class<?> c, int index) {
		switch (index) {
		case 0:
			ILoggerFactory factory = (ILoggerFactory) components.get(0);
			return factory.getLogger(c.getCanonicalName());
		default:
			return components.get(index);
		}
	}

}
