/**
 * Copyright © 2012, 2013 dr. ir. Jeroen M. Valk
 * 
 * This file is part of Badgerfish CPX. Badgerfish CPX is free software:
 * you can redistribute it and/or modify it under the terms of the GNU Lesser
 * General Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version. Badgerfish
 * CPX is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU Lesser General Public License for more details. You
 * should have received a copy of the GNU Lesser General Public License along
 * with Badgerfish CPX. If not, see <http://www.gnu.org/licenses/>.
 */

package com.googlecode.aaw.badgerfish;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

import org.slf4j.Logger;

import com.googlecode.aaw.arche.Arche;

public class Connect implements Runnable {
	static final private Arche arche = Arche.getArche(Connect.class);
	static final private Logger logger = (Logger) arche.getInstance();
	private byte[] buffer = new byte[65536];
	private InputStream input;
	private OutputStream output;
	private IOException e = null;

	public Connect(InputStream input, OutputStream output) {
		this.input = input;
		this.output = output;
	}

	@Override
	public void run() {
		if (output == null) {
			String line;
			BufferedReader reader = new BufferedReader(new InputStreamReader(input));
			try {
				while ((line = reader.readLine()) != null) {
					logger.info(line);
				}
				reader.close();
			} catch (IOException e) {
				logger.error(e.getMessage(), e);
			}
		} else {
			int n;
			try {
				while ((n = input.read(buffer)) > 0) {
					logger.debug("reading");
					output.write(buffer, 0, n);
				}
				input.close();
				output.close();
			} catch (IOException e1) {
				e = e1;
			}
		}
	}

	public void done() {
		if (e != null) {
			logger.error(e.getMessage(), e);
		}
	}
}
