/**
 * Copyright © 2012, 2013 dr. ir. Jeroen M. Valk
 * 
 * This file is part of Badgerfish CPX. Badgerfish CPX is free software:
 * you can redistribute it and/or modify it under the terms of the GNU Lesser
 * General Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version. Badgerfish
 * CPX is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU Lesser General Public License for more details. You
 * should have received a copy of the GNU Lesser General Public License along
 * with Badgerfish CPX. If not, see <http://www.gnu.org/licenses/>.
 */

package com.googlecode.aaw.badgerfish;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;

import com.googlecode.aaw.arche.Arche;

public class NodeJS {
	static final private Arche arche = Arche.getArche(Connect.class);
	static final private Logger logger = (Logger) arche.getInstance();
	static private final String cmdNodeJS = "/usr/local/bin/node";
	private final ProcessBuilder shell = new ProcessBuilder();
	private final Process process;
	private final File outputfile;

	public NodeJS(Badgerfish badgerfish, int mode) {
		byte[] script = null;
		outputfile = badgerfish.toFile();
		Map<String, String> env = shell.environment();
		env.put("PATH", env.get("PATH") + ":/usr/local/bin");
		logger.info(env.get("PATH"));
		List<String> cmd = shell.command();
		cmd.add(cmdNodeJS);
		switch (mode) {
		case 0:
			File file = badgerfish.fromFile();
			cmd.add(file.getPath());
			break;
		case 1:
			script = badgerfish.getScript().getBytes();
			cmd.add("var vm=require('vm');var fs=require('fs');var n="
					+ script.length
					+ ";var i=0;var b=new Buffer(n);while(i<n){i+=fs.readSync(0,b,i,n,null);}var sandbox={};vm.runInNewContext(b.toString('utf8'),sandbox,'test.js');");
			break;
		}
		logger.debug("Running NodeJS");
		Process process = null;
		try {
			// first try to start a local installation of NodeJS
			process = shell.start();
		} catch (IOException e) {
			// then search NodeJS in the system path
			cmd.set(0, "node");
			try {
				process = shell.start();
			} catch (IOException ee) {
				logger.error(e.getMessage(), ee);
			}
		}
		this.process = process;

		switch (mode) {
		case 1:
			OutputStream output = process.getOutputStream();
			try {
				output.write(script);
				output.close();
			} catch (IOException e) {
				logger.error(e.getMessage(), e);
			}
			break;
		}
	}

	public int waitForExitValue() {
		OutputStream output = null;
		if (outputfile == null) {
			logger.debug("Writing stdout to info log");
		} else {
			logger.debug("Writing stdout to file: "
					+ outputfile.getAbsolutePath());
			try {
				output = new FileOutputStream(outputfile);
			} catch (FileNotFoundException e) {
				logger.error(e.getMessage(), e);
			}
		}
		Connect connect = null;
		connect = new Connect(process.getInputStream(), output);
		Thread t = new Thread(connect);
		t.start();
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				process.getErrorStream()));
		try {
			String line;
			while ((line = reader.readLine()) != null) {
				logger.error(line);
			}
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
		try {
			t.join();
		} catch (InterruptedException e) {
			logger.error(e.getMessage(), e);
		}
		connect.done();
		try {
			process.waitFor();
		} catch (InterruptedException e) {
			logger.error(e.getMessage(), e);
		}
		return process.exitValue();
	}
}
