/**
 * Copyright © 2012, 2013 dr. ir. Jeroen M. Valk
 * 
 * This file is part of Badgerfish CPX. Badgerfish CPX is free software:
 * you can redistribute it and/or modify it under the terms of the GNU Lesser
 * General Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version. Badgerfish
 * CPX is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU Lesser General Public License for more details. You
 * should have received a copy of the GNU Lesser General Public License along
 * with Badgerfish CPX. If not, see <http://www.gnu.org/licenses/>.
 */

package com.googlecode.aaw.init;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OperatingSystemCheck {
	public static final int UNKNOWN = -1;
	public static final int CYGWIN = 0;
	public static final int DARWIN = 1;
	public static final int LINUX = 2;

	public static int getOS() {
		Logger logger = LoggerFactory.getLogger(OperatingSystemCheck.class);
		int ostype = -1;
		Process process = null;
		ProcessBuilder shell = new ProcessBuilder();
		List<String> cmd = shell.command();
		cmd.add("uname");
		cmd.add("-s");
		try {
			process = shell.start();
		} catch (IOException e) {
			process = null;
		}

		if (process != null) {
			try {
				byte[][] os = { "Cygwin".getBytes("UTF8"),
						"Darwin".getBytes("UTF8"), "Linux".getBytes("UTF8") };
				byte[] buffer = new byte[1024];
				process.getInputStream().read(buffer);
				for (int i = 0; i < os.length; ++i) {
					int j;
					for (j = 0; j < os[i].length; ++j) {
						if (buffer[j] != os[i][j]) {
							break;
						}
					}
					if (j == os[i].length) {
						ostype = i;
						break;
					}
				}
			} catch (UnsupportedEncodingException e) {
				logger.error(e.getMessage(), e);
				System.exit(1);
			} catch (IOException e) {
				logger.error(e.getMessage(), e);
			}
		}
		return ostype;
	}
}
